package it.sc.xmpplugin.main;

import it.sc.xmpplugin.jpeg.JpegPicture;
import it.sc.xmpplugin.jpeg.JpegWriter;
import it.sc.xmpplugin.jpeg.XMPManager;
import it.sc.xmpplugin.presentation.FormData;
import it.sc.xmpplugin.utils.FileUtils;
import it.sc.xmpplugin.xmp.XMPSchemaBasic;
import it.sc.xmpplugin.xmp.XMPSchemaDublinCore;
import it.sc.xmpplugin.xmp.XMPSchemaImageJ;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Calendar;
import java.util.List;

public class GestioneXMP {

    public static FormData readXMPFromJPEG(String path) {

        FormData formData = null;
        XMPSchemaDublinCore dc = null;
        try {

            JpegPicture picture = new JpegPicture(path);
            XMPManager xm = new XMPManager(picture);
            //if(picture.getJpegSegmentData().getSegment(JpegSegmentReader.SEGMENT_APP1) == null)
            //    return null;
            dc = xm.getXmpXML().getDublinCoreSchema();
            XMPSchemaImageJ imageSchema = xm.getXmpXML().getImageJSchema();
            if (dc != null) {
                formData = new FormData();
                formData.setAutori(dc.getCreators());
                formData.setDescrizione(dc.getDescription());
                formData.setTitolo(dc.getTitle());
                formData.setSoggetti(dc.getSubjects());
                formData.setEditori(dc.getPublishers());
                formData.setDate(dc.getDates());

                formData.setTipi(dc.getTypes());
                formData.setFormato(dc.getFormat());
                formData.setIdentifier(dc.getIdentifier());
                formData.setSource(dc.getSource());

                formData.setLinguaggi(dc.getLanguages());
                formData.setRelazioni(dc.getRelationships());
                formData.setContributi(dc.getContributors());

                formData.setCopertura(dc.getCoverage());
                formData.setDiritti(dc.getRights());

                formData.setCitta(imageSchema.getCity());
                formData.setDataCreazione(imageSchema.getDateCreated());
                formData.setNazione(imageSchema.getCountry());
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
        return formData;

    }

    public static boolean writeXMPFromJPEG(String inFile, String outFile, FormData formData, boolean modifica) {

        try {
            System.out.println("inFile:" + inFile);
            System.out.println("outFile:" + outFile);

            JpegPicture picture = new JpegPicture(inFile);
            XMPManager xm = new XMPManager(picture);
            XMPSchemaBasic tmp = xm.getXmpXML().getBasicSchema();
            if (tmp == null) {
                tmp = xm.getXmpXML().addBasicSchema();
            }
            if (!modifica) {
                tmp.setRating(5);
                tmp.setRating(4);
            }
            XMPSchemaDublinCore dc = xm.getXmpXML().getDublinCoreSchema();
            if (dc == null) {
                dc = xm.getXmpXML().addDublinCoreSchema();
            }

            dc.removeSubjectTag();
            System.out.println("estrazione autori");

            // rimozione di tutti gli elementi
            List<String> autoriToDel = dc.getCreators();
            if (autoriToDel != null) {
                for (String aut : autoriToDel) {
                    dc.removeCreator(aut);
                }
            }
            // aggiungo gli elementi inseriti nell'interfaccia
            List<String> autori = formData.getAutori();
            for (String aut : autori) {
                dc.addCreator(aut);
            }

            // rimozione di tutti gli elementi
            List<String> soggettiToDel = dc.getSubjects();
            if (soggettiToDel != null) {
                for (String sog : soggettiToDel) {
                    dc.removeSubject(sog);
                }
            }
            System.out.println("estrazione soggetti");
            List<String> soggetti = formData.getSoggetti();
            for (String sogg : soggetti) {
                dc.addSubject(sogg);
            }

            if (formData.getTitolo() != null && !formData.getTitolo().equals("")) {
                dc.setTitle(formData.getTitolo());
            } else {
                dc.setTitle("");
            }
            if (formData.getDescrizione() != null && !formData.getDescrizione().equals("")) {
                dc.setDescription(formData.getDescrizione());
            } else {
                dc.setDescription("");
            }

            System.out.println("estrazione editori");
            // rimozione di tutti gli elementi
            List<String> editoriToDel = dc.getPublishers();
            if (editoriToDel != null) {
                for (String edt : editoriToDel) {
                    dc.removePublisher(edt);
                }
            }
            List<String> editori = formData.getEditori();
            for (String edit : editori) {
                dc.addPublisher(edit);
            }

            System.out.println("estrazione date");
            // rimozione di tutti gli elementi
            List<Calendar> dateToDel = dc.getDates();
            if (dateToDel != null) {
                for (Calendar cal : dateToDel) {
                    dc.removeDate(cal);
                }
            }
            List<Calendar> date = formData.getDate();
            if (date != null && date.size() > 0) {
                for (Calendar dat : date) {
                    dc.addDate(dat);
                }
            }

            System.out.println("estrazione tipi");
            // rimozione di tutti gli elementi
            List<String> typeToDel = dc.getTypes();
            if (typeToDel != null) {
                for (String type : typeToDel) {
                    dc.removeType(type);
                }
            }
            List<String> tipi = formData.getTipi();
            for (String tip : tipi) {
                dc.addType(tip);
            }

            System.out.println("estrazione formato, id e source");
            if (formData.getFormato() != null && !formData.getFormato().equals("")) {
                dc.setFormat(formData.getFormato());
            } else {
                dc.setFormat("");
            }
            if (formData.getIdentifier() != null && !formData.getIdentifier().equals("")) {
                dc.setIdentifier(formData.getIdentifier());
            } else {
                dc.setIdentifier("");
            }
            if (formData.getSource() != null && !formData.getSource().equals("")) {
                dc.setSource(formData.getSource());
            } else {
                dc.setSource("");
            }

            System.out.println("estrazione linguaggi");
            List<String> langToDel = dc.getLanguages();
            if (langToDel != null) {
                for (String lan : langToDel) {
                    dc.removeLanguage(lan);
                }
            }
            List<String> ling = formData.getLinguaggi();
            for (String l : ling) {
                dc.addLanguage(l);
            }

            System.out.println("estrazione relazioni");
            List<String> relToDel = dc.getRelationships();
            if (relToDel != null) {
                for (String rel : relToDel) {
                    dc.removeRelation(rel);
                }
            }
            List<String> relat = formData.getRelazioni();
            for (String rel : relat) {
                dc.addRelation(rel);
            }

            System.out.println("estrazione contributi");
            List<String> contrToDel = dc.getContributors();
            if (contrToDel != null) {
                for (String con : contrToDel) {
                    dc.removeContributor(con);
                }
            }
            List<String> contr = formData.getContributi();
            if (contr != null) {
                for (String con : contr) {
                    dc.addContributor(con);
                }
            }

            if (formData.getCopertura() != null && !formData.getCopertura().equals("")) {
                dc.setCoverage(formData.getCopertura());
            } else {
                dc.setCoverage("");
            }
            if (formData.getDiritti() != null && !formData.getDiritti().equals("")) {
                dc.setRights(formData.getDiritti());
            } else {
                dc.setRights("");
            }

            XMPSchemaImageJ ps = xm.getXmpXML().getImageJSchema();
            if (ps == null) {
                ps = xm.getXmpXML().addImageJSchema();
            }

            System.out.println("estrazione campi aggiuntivi");
            if (formData.getCitta() != null && !formData.getCitta().equals("")) {
                ps.setCity(formData.getCitta());
            } else {
                ps.setCity("");
            }
            if (formData.getStato() != null && !formData.getStato().equals("")) {
                ps.setState(formData.getStato());
            } else {
                ps.setState("");
            }
            if (formData.getNazione() != null && !formData.getNazione().equals("")) {
                ps.setCountry(formData.getNazione());
            } else {
                ps.setCountry("");
            }
            if (formData.getDataCreazione() != null && !formData.getDataCreazione().equals("")) {
                ps.setDateCreated(formData.getDataCreazione());
            } else {
                ps.setDateCreated("");
            }

            xm.saveXMPintoAPP1();

            File newFile = new File(outFile);
            FileOutputStream os = new FileOutputStream(newFile);
            BufferedInputStream is = new BufferedInputStream(new FileInputStream(inFile));
            JpegWriter.writeJpegFile(picture.getJpegSegmentData(), os, is);


        } catch (Exception e) {
            e.printStackTrace();
        }
        return true;
    }

    public static boolean modifyXMPFromJPEG(String inFile, String outFile, FormData formData) {

        try {

            // copia file di origine
            String backupFile = "temp.jpg";
            String folder = outFile.substring(0, outFile.lastIndexOf("\\"));
            FileUtils.copyfile(inFile, folder + "\\" + backupFile);

            FileUtils.cancellaFile(inFile);
            inFile = folder + "\\" + backupFile;

            writeXMPFromJPEG(inFile, outFile, formData, true);

            FileUtils.cancellaFile(inFile);

        } catch (Exception e) {
            e.printStackTrace();
        }
        return true;
    }

    private static boolean containsElem(Object[] elemList, Object elem) {
        for (int i = 0; i < elemList.length; i++) {
            Object currElem = elemList[i];
            if (currElem instanceof String) {
                if (currElem.toString().equals(elem)) {
                    return true;
                }
            } else if (currElem instanceof Calendar) {
                Calendar curCal = (Calendar) currElem;
                Calendar cal = (Calendar) elem;
                if (curCal.getTime() == cal.getTime()) {
                    return true;
                }
            }
        }
        return false;
    }

    public static String getRDFStructure(String pathFile) {
        String rdf = "";
        try {

            JpegPicture picture = new JpegPicture(pathFile);
            XMPManager xm = new XMPManager(picture);
            byte[] xmpByte = xm.getXMPasBytes();
            if (xmpByte != null) {
                for (int i = 0; i < xmpByte.length; i++) {
                    rdf += ((char) xmpByte[i]);
                }
            } else {
                System.out.print("No data.");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return rdf;
    }
}
